// ****************************************************************************************************************
// controller lib (internal functions)
// ****************************************************************************************************************

static void _reset_si(void)
{

  // clear si regs
  *(volatile unsigned long*)0xCC006400 = 0x00000000; // SIC0OUTBUF - SI Channel 0 Output Buffer
  *(volatile unsigned long*)0xCC00640c = 0x00000000;
  *(volatile unsigned long*)0xCC006418 = 0x00000000;
  *(volatile unsigned long*)0xCC006424 = 0x00000000;

  *(volatile unsigned long*)0xCC006404 = 0x00000000; // SIC1INBUFH - SI Channel 0 Input Buffer High
  *(volatile unsigned long*)0xCC006410 = 0x00000000;
  *(volatile unsigned long*)0xCC00641c = 0x00000000;
  *(volatile unsigned long*)0xCC006428 = 0x00000000;

  *(volatile unsigned long*)0xCC006408 = 0x00000000; // SIC1INBUFH
  *(volatile unsigned long*)0xCC006414 = 0x00000000;
  *(volatile unsigned long*)0xCC006420 = 0x00000000;
  *(volatile unsigned long*)0xCC00642c = 0x00000000;

  *(volatile unsigned long*)0xCC006430 = 0x00000000; // SIPOLL Config
  *(volatile unsigned long*)0xCC006434 = 0x00000000; // SICOMCSR - SI Communication Control Status Register (command)
  *(volatile unsigned long*)0xCC006438 = 0x00000000; // SISR - SI Status Register (channel select & status2)

  *(volatile unsigned long*)0xCC006480 = 0x00000000;
  *(volatile unsigned long*)0xCC006484 = 0x00000000;
  *(volatile unsigned long*)0xCC006488 = 0x00000000;
  *(volatile unsigned long*)0xCC00648c = 0x00000000;

  *(volatile unsigned long*)0xCC006490 = 0x00000000;
  *(volatile unsigned long*)0xCC006494 = 0x00000000;
  *(volatile unsigned long*)0xCC006498 = 0x00000000;
  *(volatile unsigned long*)0xCC00649c = 0x00000000;

  *(volatile unsigned long*)0xCC0064a0 = 0x00000000;
  *(volatile unsigned long*)0xCC0064a4 = 0x00000000;
  *(volatile unsigned long*)0xCC0064a8 = 0x00000000;
  *(volatile unsigned long*)0xCC0064ac = 0x00000000;

}

void _wait_tcinit(void)
{
unsigned long id;

    do
    {
        id=*(volatile unsigned long*)(0xCC006434);
    } while((id&(1<<31))==0);
    *(volatile unsigned long*)0xCC006434 |= (1<<31);        // mask irq
}

/*
  - must restore previous polling state etc after calling this function
*/
unsigned long pad_getid(int pad)
{
unsigned long i;
unsigned long id;

  _reset_si();

  *(volatile unsigned long*)0xCC006430 = 0x00000000;
  *(volatile unsigned long*)(0xCC006400+(12*pad))  = 0x00000000; // command 0x00000000
  *(volatile unsigned long*)0xCC006438 = 0x80000000;
  *(volatile unsigned long*)0xCC006434 = (
        //0xc0010101
        (1<<31)|        // clear irq
        (1<<30)|        // mask irq
        (1<<28)|        // clear irq
        (0<<27)|        // mask irq
        (0<<25)|        // channel number
        (0<<24)|        // channel enable
        (0x01<<16)|     // outlen
        (0x8<8)|      // inlen
        (pad<<1)|       // channel
        (1<<0)          // TSTART
        );

_wait_tcinit();

    id=*(volatile unsigned long*)(0xCC006480);

    return(id);
}

/*
  restores standard polling channels for all 4 channels
*/
static void _set_std_polling(ControllerInfo *Controllers)
{
int i,id;
unsigned long padbits=0;


    for(i=0;i<4;i++)
    {
          switch(Controllers[i].id)
          {
                case CID_STD:
                      *(volatile unsigned long*)(0xCC006400+(12*i))  = //0x00400300;
                      Controllers[i].command;
          padbits |= (1<<((7-i))); /* x-times=0x47, y-times=0x02 */
                break;
                case CID_KEYB:
                      *(volatile unsigned long*)(0xCC006400+(12*i))  = //0x00540000;
                      Controllers[i].command;
          padbits |= (1<<((7-i))); /* x-times=0x47, y-times=0x02 */
                break;
          }
    }
          *(volatile unsigned long*)0xCC006430 = 0x00f70200|padbits;
          *(volatile unsigned long*)0xCC006438 = 0x80000000;
*(volatile unsigned long*)0xCC006434 = (
        //0xc0010101
        (1<<31)|        // clear irq
        (1<<30)|        // mask irq
        (0<<28)|        // clear irq
        (0<<27)|        // mask irq
        (0<<25)|        // channel number
        (0<<24)|        // channel enable
        (1<<16)|     // outlen
        (8<<8)|      // inlen
        (0<<1)|       // channel
        (1<<0)          // TSTART
        );

    _wait_tcinit();

}

// ****************************************************************************************************************
// controller lib (public functions)
// ****************************************************************************************************************

int ControllerRead(ControllerInfo *info)
{
int i;
unsigned long iw1,iw2;
int err=0;

    for(i=0;i<4;i++)
    {
        iw1=*(volatile unsigned long*)(0xCC006404+(12*i));
        iw2=*(volatile unsigned long*)(0xCC006408+(12*i));
        info[i].raw1=iw1;
        info[i].raw.word[0]=iw1;
        info[i].raw2=iw2;
        info[i].raw.word[1]=iw2;

        switch(info[i].id)
        {
            case CID_KEYB:
                info[i].keyb.key[0]=(iw1>>12)&0xff;
                info[i].keyb.key[1]=(iw1>>4)&0xff;
                info[i].keyb.key[2]=(iw1<<4)&0xf0;
                info[i].keyb.key[2]|=(iw2>>28)&0x0f;
                //info[i].raw.errstat=iw1>>31;
                //info[i].raw.errlatch=(iw1>>30)&1;
                info[i].raw.errstat=0;
                info[i].raw.errlatch=0;
                break;
            case CID_STD:
                break;
            case CID_GBA:
                break;
            default:
                break;
        }
    }
    return(err);
}


void ControllerRumble(ControllerInfo *info,int chan,int onoff)
{
        switch(info[chan].id)
        {
            case CID_STD:
                if(onoff)
                {
                    info[chan].command|=1;
                }
                else
                {
                    info[chan].command&=~1;
                }
                //*(volatile unsigned long*)(0xCC006400+(12*chan)) = info[chan].command;
                //*(volatile unsigned long*)0xCC006434 = 0x80010301;
                //*(volatile unsigned long*)0xCC006438 = 0x80000000;
                _set_std_polling(info);
                break;
            case CID_KEYB:
                break;
            case CID_GBA:
                break;
            default:
                break;
        }
}

void ControllerReInit(ControllerInfo *Controllers)
{
unsigned long id;
unsigned long i;

    _reset_si();

    /*
        get ID from all four controllers
    */
    for(i=0;i<4;i++)
    {
          id=pad_getid(i);
          Controllers[i].status=(id>>8)&0xff;
          id>>=16;

          /* set std command for polling */
          if(Controllers[i].id!=id)
          {
              Controllers[i].id=id;
              switch(Controllers[i].id)
              {
                    case CID_STD:
                      Controllers[i].command=0x00400300;
                      break;
                    case CID_KEYB:
                      Controllers[i].command=0x00540000;
                      break;
                    default:
                      Controllers[i].command=0x00000000;
                      break;
              }
          }
     }

     _set_std_polling(Controllers);
}


/* main init routine, must be called once before any of the other functions */
void ControllerInit(ControllerInfo *Controllers)
{
unsigned long id;
unsigned long i;

    _reset_si();

    /*
        get ID from all four controllers
    */
    for(i=0;i<4;i++)
    {
          id=pad_getid(i);
          Controllers[i].status=(id>>8)&0xff;
          Controllers[i].id=id>>16;

          /* set std command for polling */
          switch(Controllers[i].id)
          {
                case CID_STD:
                  Controllers[i].command=0x00400300;
                  break;
                case CID_KEYB:
                  Controllers[i].command=0x00540000;
                  break;
                default:
                  Controllers[i].command=0;
                  break;
          }

#ifdef DEBUG
          gotoxy(0,2+(6*i));
          cprintf("id:%08x\n",id);
          p_04(i);p_80();p_6500();p_34();p_38(i);
#endif
     }

     _set_std_polling(Controllers);

#ifdef DEBUG
  gotoxy(0,26);
  p_04(0);p_04(1);p_04(2);p_04(3);
#endif

}

