
#ifndef __CONTROLLER_H__
#define __CONTROLLER_H__

typedef struct
{
  union
  {
    unsigned char errstat : 1;
    unsigned char errlatch : 1;
    unsigned long word[2];
  };
} RAWStatus;
typedef struct
{
    unsigned char errstat : 1;
    unsigned char errlatch : 1;
    unsigned char key[3];
} KEYStatus;
typedef struct
{
  unsigned char errstat : 1;
  unsigned char errlatch : 1;
  unsigned char na1 : 1;
  unsigned char s : 1 ;
  unsigned char y : 1 ;
  unsigned char x : 1 ;
  unsigned char b : 1 ;
  unsigned char a : 1 ;

  unsigned char na2 : 1;
  unsigned char lt : 1 ;
  unsigned char rt : 1 ;
  unsigned char z : 1 ;
  unsigned char u : 1 ;
  unsigned char d : 1 ;
  unsigned char r : 1 ;
  unsigned char l : 1 ;

  unsigned char ax ;
  unsigned char ay ;
  unsigned char cx ;
  unsigned char cy ;
  unsigned char al ;
  unsigned char ar ;
} STDStatus;

typedef struct
{
  unsigned long id;
  unsigned long status;
  unsigned long command;
  unsigned long raw1,raw2;
  union
  {
    RAWStatus raw;
    STDStatus pad;
    KEYStatus keyb;
  };
} ControllerInfo;

/*

Controller polling commands:

    any:
     0x00000000 - recieve ID and status

    std controller:
     0x00400000 - poll controller
     0x00000300 - ?
     0x00000001 - rumble on

    keyboard:
     0x00540000

Controller polling answers:

Max-Play,
Original
Wavebird,
RecieverWavebird,
GBA Controller
Keyboard
                                    Description
---16bit controller id-------
24  0   0    1   1    0    0       wireless (1: wireless Controller)
23  0   0    0   1    0    0       wireless receive (0: not wireless 1: wireless)
22  0   0    1   1    0    0       Rumble Motor (0: available 1: not available)
21  0 0 0 0  0 a 0 e  0 0  0 0     Type of Controller (always 0 ?)
20  1   1    1   1    0    1       Type of Controller (0: N64 1: Dolphin)
19  0   0    0   0    0    0       wireless type (0:IF 1:RF)
18  0   0    0   0    0    0       wireless state (0: variable 1: fixed)
17  1 9 1 9  0 8 1 9  0 0  0 8     0: non standard Controller 1: Dolphin Standard Controller

16  0   0    0   0    0    0
15  0   0    0   1    0    0
14  0   0    0   1    0    1       wireless origin (0:invalid 1:valid)
13  0 0 0 0  0 0 0 6  0 0  0 2     wireless fix id (0:not fixed 1:fixed)
12  0   0    0   0    0    0       wireless type - 0:normal 1: non-controller (?)
11  0   0    0   0    1    0       wireless type - 0:normal 1: lite controller
10  0   0    0   0    0    0       wireless type -
9   0 0 0 0  0 0 0 0  0 4  0 0     wireless type -
---8bit controller status- -----
8   0   0    0   0    0
7   0   0    0   1    0            ?
6   1   1    0   0    0            ?
5   0 2 0 2  0 0 1 5  0 0
4   0   0    0   0    1            rumble motor running ?
3   0   0    0   0    0
2   0   0    0   1    0            ?
1   0 0 0 0  0 0 0 2  0 8          ?

0 [Stopbit]

*/

/* controller ids */
#define CID_STD             0x0900
#define CID_KEYB            0x0820
#define CID_GBA             0x0004
#define CID_GBA_NA          0x0800
#define CID_WAVEBIRD        0xa800
#define CID_WAVEBIRD_RCV    0xe960

/* main init routine, must be called once before any of the other functions */
void ControllerInit(ControllerInfo *Controllers);
void ControllerReInit(ControllerInfo *Controllers);
void ControllerRumble(ControllerInfo *Controllers,int chan,int onoff);
int ControllerRead(ControllerInfo *Controllers);

#endif
